/*
    AWFFull - A Webalizer Fork, Full o' features
    
    $Id: parser.c 149 2006-01-24 11:12:26Z steve $

    Copyright (C) 1997-2001  Bradford L. Barrett (brad@mrunix.net)
    Copyright (C) 2004, 2005 by Stephen McInerney (spm@stedee.id.au)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version, and provided that the above
    copyright and permission notice is included with all distributed
    copies of this or derived software.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

    This software uses the gd graphics library, which is copyright by
    Quest Protein Database Center, Cold Spring Harbor Labs.  Please
    see the documentation supplied with the library for additional
    information and license terms, or visit www.boutell.com/gd/ for the
    most recent version of the library and supporting documentation.
*/


/* Parser Module
 * All code related to parsing log lines and individual records, move here
 */

/*********************************************/
/* STANDARD INCLUDES                         */
/*********************************************/

#include "awffull.h"                            /* main header              */

/* internal function prototypes */
void fmt_logrec(char *);
int parse_record_web(char *);
int parse_record_ftp(char *);
int parse_record_squid(char *);

void re_compile_failed(int, const char *, char *);      /* Display a failed RE Compile & where */
void re_check_errors(int);                      /* After an RE check, deal with any errors */
void error_substring_extract(int, int);         /* Error when we fail on getting a substring */

pcre *cmp_log_regexp = NULL;                    /* Main compiled RE */

//pcre_extra *studied_log_regexp = NULL;

/*********************************************/
/* FMT_LOGREC - terminate log fields w/zeros */
/*********************************************/

void
fmt_logrec(char *buffer)
{
    char *cp = buffer;
    int q = 0, b = 0, p = 0;

    while (*cp != '\0') {
        /* break record up, terminate fields with '\0' */
        switch (*cp) {
        case ' ':
            if (b || q || p)
                break;
            *cp = '\0';
            break;
        case '"':
            q ^= 1;
            break;
        case '[':
            if (q)
                break;
            b++;
            break;
        case ']':
            if (q)
                break;
            if (b > 0)
                b--;
            break;
        case '(':
            if (q)
                break;
            p++;
            break;
        case ')':
            if (q)
                break;
            if (p > 0)
                p--;
            break;
        }
        cp++;
    }
}

/*********************************************/
/* PARSE_RECORD - uhhh, you know...          */
/*********************************************/

int
parse_record(char *buffer)
{
    char log_regexp[MAX_RE_LENGTH] = PATTERN_NCSA;      /* Hold the primary log splitter RE */
    const char *error;                          /* RE error pointer, offset */
    int erroffset;                              /* RE error value */

    /* clear out structure */
    memset(&log_rec, 0, sizeof(struct log_struct));

    /* call appropriate handler */
    switch (log_type) {
    default:
    case LOG_CLF:
        if (cmp_log_regexp == NULL) {
            cmp_log_regexp = pcre_compile(log_regexp, 0, &error, &erroffset, NULL);
            VPRINT(VERBOSE2, "PCRE: Compile Main%s", "\n")
                if (cmp_log_regexp == NULL) {
                re_compile_failed(erroffset, error, log_regexp);
            }
/*
			studied_log_regexp = pcre_study(cmp_log_regexp, 0, &error);
			if (studied_log_regexp == NULL) {
				ERRVPRINT(VERBOSE0, "Failed to study%s", "\n");
			}
*/
        }

        return parse_record_web(buffer);
        break;                                  /* clf   */
    case LOG_FTP:
        return parse_record_ftp(buffer);
        break;                                  /* ftp   */
    case LOG_SQUID:
        return parse_record_squid(buffer);
        break;                                  /* squid */
    }
}

/*********************************************/
/* PARSE_RECORD_FTP - ftp log handler        */
/*********************************************/

int
parse_record_ftp(char *buffer)
{
    int size;
    int i, j;
    char *cp1, *cp2, *cpx, *cpy, *eob;

    size = strlen(buffer);                      /* get length of buffer        */
    eob = buffer + size;                        /* calculate end of buffer     */
    fmt_logrec(buffer);                         /* seperate fields with \0's   */

    /* Start out with date/time       */
    cp1 = buffer;
    while (*cp1 != 0 && cp1 < eob)
        cp1++;
    while (*cp1 == 0)
        cp1++;
    cpx = cp1;                                  /* save month name */
    while (*cp1 != 0 && cp1 < eob)
        cp1++;
    while (*cp1 == 0)
        cp1++;
    i = atoi(cp1);                              /* get day number  */
    while (*cp1 != 0 && cp1 < eob)
        cp1++;
    while (*cp1 == 0)
        cp1++;
    cpy = cp1;                                  /* get timestamp   */
    while (*cp1 != 0 && cp1 < eob)
        cp1++;
    while (*cp1 == 0)
        cp1++;
    j = atoi(cp1);                              /* get year        */

    /* minimal sanity check */
    if (*(cpy + 2) != ':' || *(cpy + 5) != ':')
        return 0;
    if (j < 1990 || j > 2100)
        return 0;
    if (i < 1 || i > 31)
        return 0;

    /* format date/time field         */
    sprintf(log_rec.datetime, "[%02d/%s/%4d:%s -0000]", i, cpx, j, cpy);

    /* skip seconds... */
    while (*cp1 != 0 && cp1 < eob)
        cp1++;
    while (*cp1 == 0)
        cp1++;
    while (*cp1 != 0 && cp1 < eob)
        cp1++;
    while (*cp1 == 0)
        cp1++;

    /* get hostname */
    cp2 = log_rec.hostname;
    strncpy(cp2, cp1, MAXHOST - 1);

    /* get filesize */
    while (*cp1 != 0 && cp1 < eob)
        cp1++;
    while (*cp1 == 0)
        cp1++;
    if (*cp1 < '0' || *cp1 > '9')
        log_rec.xfer_size = 0;
    else
        log_rec.xfer_size = strtoul(cp1, NULL, 10);

    /* URL stuff */
    while (*cp1 != 0 && cp1 < eob)
        cp1++;
    while (*cp1 == 0)
        cp1++;
    cpx = cp1;
    /* get next field for later */
    while (*cp1 != 0 && cp1 < eob)
        cp1++;
    while (*cp1 == 0)
        cp1++;
    if (strlen(cpx) > MAXURL - 20)
        *(cpx + (MAXURL - 20)) = 0;

    /* skip next two */
    while (*cp1 != 0 && cp1 < eob)
        cp1++;
    while (*cp1 == 0)
        cp1++;
    while (*cp1 != 0 && cp1 < eob)
        cp1++;
    while (*cp1 == 0)
        cp1++;

    /* fabricate an appropriate request string based on direction */
    if (*cp1 == 'i')
        sprintf(log_rec.url, "\"POST %s HTTP/1.0\"", cpx);
    else
        sprintf(log_rec.url, "\"GET %s HTTP/1.0\"", cpx);

    if (cp1 < eob)
        cp1++;
    if (cp1 < eob)
        cp1++;
    while (*cp1 != 0 && cp1 < eob)
        cp1++;
    if (cp1 < eob)
        cp1++;
    cp2 = log_rec.ident;
    while (*cp1 != 0 && cp1 < eob)
        *cp2++ = *cp1++;
    *cp2 = '\0';

    /* return appropriate response code */
    log_rec.resp_code = (*(eob - 2) == 'i') ? 206 : 200;

    return 1;
}


/*********************************************
 * PARSE_RECORD_WEB - web log handler        *
 * parse with pcre							 *
 *********************************************/
int
parse_record_web(char *buffer)
{
    int ovector[OVECCOUNT];                     /* RE substring offsets array */
    int rc;                                     /* RE Check return value */
    int copy_substr_rtn;                        /* RE Check return from pcre_copy_substring */

    int buffer_length;

    char tmp_status[5];
    char tmp_bytes[20];

    buffer_length = (int) strlen(buffer);
//    rc = pcre_exec (cmp_log_regexp, studied_log_regexp, buffer, buffer_length, 0, 0, ovector, OVECCOUNT);
    rc = pcre_exec(cmp_log_regexp, NULL, buffer, buffer_length, 0, 0, ovector, OVECCOUNT);
    /* check for RE matching errors */
    if (rc < 0) {
        re_check_errors(rc);
        return (0);
    }

    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_ADDRESS, log_rec.hostname, MAXHOST);
    if (copy_substr_rtn < 0) {
        error_substring_extract(copy_substr_rtn, LF_NCSA_ADDRESS);
        return (0);
    }
    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_USER, log_rec.ident, MAXIDENT);
    if (copy_substr_rtn < 0) {
        error_substring_extract(copy_substr_rtn, LF_NCSA_USER);
        return (0);
    }
    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_DATE_TIME, log_rec.datetime, SIZE_DATE_TIME);
    if (copy_substr_rtn < 0) {
        error_substring_extract(copy_substr_rtn, LF_NCSA_DATE_TIME);
        return (0);
    }

    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_URL, log_rec.url, MAXURL);
    if (copy_substr_rtn < 0) {
        error_substring_extract(copy_substr_rtn, LF_NCSA_URL);
        return (0);
    }

    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_STATUS, tmp_status, 5);
    if (copy_substr_rtn < 0) {
        error_substring_extract(copy_substr_rtn, LF_NCSA_STATUS);
        return (0);
    }
    log_rec.resp_code = atoi(tmp_status);

    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_BYTES, tmp_bytes, 20);
    if (copy_substr_rtn < 0) {
        error_substring_extract(copy_substr_rtn, LF_NCSA_BYTES);
        return (0);
    }
    log_rec.xfer_size = strtoul(tmp_bytes, NULL, 10);

    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_REFERER, log_rec.refer, MAXREF);
    if (copy_substr_rtn < 0) {
        error_substring_extract(copy_substr_rtn, LF_NCSA_REFERER);
        return (0);
    }
    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_BROWSER, log_rec.agent, MAXAGENT);
    if (copy_substr_rtn < 0) {
        error_substring_extract(copy_substr_rtn, LF_NCSA_BROWSER);
        return (0);
    }
    return (1);
}


/*********************************************/
/* PARSE_RECORD_SQUID - squid log handler    */
/*********************************************/

int
parse_record_squid(char *buffer)
{
    int size;
    time_t i;
    char *cp1, *cp2, *cpx, *eob, *eos;

    size = strlen(buffer);                      /* get length of buffer        */
    eob = buffer + size;                        /* calculate end of buffer     */
    fmt_logrec(buffer);                         /* seperate fields with \0's   */

    /* date/time */
    cp1 = buffer;
    i = atoi(cp1);                              /* get timestamp */

    /* format date/time field */
    strftime(log_rec.datetime, sizeof(log_rec.datetime), "[%d/%b/%Y:%H:%M:%S -0000]", localtime(&i));

    while (*cp1 != 0 && cp1 < eob)
        cp1++;
    while (*cp1 == 0)
        cp1++;

    /* skip request size */
    while (*cp1 != 0 && cp1 < eob)
        cp1++;
    while (*cp1 == 0)
        cp1++;

    /* HOSTNAME */
    cpx = cp1;
    cp2 = log_rec.hostname;
    eos = (cp1 + MAXHOST) - 1;
    if (eos >= eob)
        eos = eob - 1;

    while ((*cp1 != '\0') && (cp1 != eos))
        *cp2++ = *cp1++;
    *cp2 = '\0';
    if (*cp1 != '\0') {
        if (verbose) {
            fprintf(stderr, "%s", msg_big_host);
            if (debug_mode)
                fprintf(stderr, ": %s\n", cpx);
            else
                fprintf(stderr, "\n");
        }
        while (*cp1 != '\0')
            cp1++;
    }
    if (cp1 < eob)
        cp1++;

    /* skip cache status */
    while (*cp1 != 0 && cp1 < eob && *cp1 != '/')
        cp1++;
    cp1++;

    /* response code */
    log_rec.resp_code = atoi(cp1);
    while (*cp1 != 0 && cp1 < eob)
        cp1++;
    while (*cp1 == 0)
        cp1++;

    /* xfer size */
    if (*cp1 < '0' || *cp1 > '9')
        log_rec.xfer_size = 0;
    else
        log_rec.xfer_size = strtoul(cp1, NULL, 10);

    while (*cp1 != 0 && cp1 < eob)
        cp1++;
    while (*cp1 == 0)
        cp1++;

    /* HTTP request type */
    cpx = cp1;
    cp2 = log_rec.url;
    *cp2++ = '\"';
    eos = (cp1 + MAXURL - 1);
    if (eos >= eob)
        eos = eob - 1;

    while ((*cp1 != '\0') && (cp1 != eos))
        *cp2++ = *cp1++;
    *cp2 = '\0';
    if (*cp1 != '\0') {
        if (verbose) {
            fprintf(stderr, "%s", msg_big_req);
            if (debug_mode)
                fprintf(stderr, ": %s\n", cpx);
            else
                fprintf(stderr, "\n");
        }
        while (*cp1 != '\0')
            cp1++;
    }
    if (cp1 < eob)
        cp1++;

    *cp2++ = ' ';

    /* HTTP URL requested */
    cpx = cp1;
    eos = (cp1 + MAXURL - 1);
    if (eos >= eob)
        eos = eob - 1;

    while ((*cp1 != '\0') && (cp1 != eos))
        *cp2++ = *cp1++;
    *cp2 = '\0';
    if (*cp1 != '\0') {
        if (verbose) {
            fprintf(stderr, "%s", msg_big_req);
            if (debug_mode)
                fprintf(stderr, ": %s\n", cpx);
            else
                fprintf(stderr, "\n");
        }
        while (*cp1 != '\0')
            cp1++;
    }
    if (cp1 < eob)
        cp1++;

    *cp2++ = '\"';

    /* IDENT (authuser) field */
    cpx = cp1;
    cp2 = log_rec.ident;
    eos = (cp1 + MAXIDENT - 1);
    if (eos >= eob)
        eos = eob - 1;

    while (*cp1 == ' ')
        cp1++;                                  /* skip white space */

    while ((*cp1 != ' ' && *cp1 != '\0') && (cp1 < eos))
        *cp2++ = *cp1++;

    *cp2-- = '\0';

    if (cp1 >= eob)
        return 0;

    /* strip trailing space(s) */
    while (*cp2 == ' ')
        *cp2-- = '\0';

    /* we have no interest in the remaining fields */
    return 1;
}


/************************************************************************
 * parse_check_not_page                                                 *
 *                                                                      *
 * Given a URL (field from a log line)                                  *
 *   determine if this URL is a page or not                             *
 *                                                                      *
 * This function is the opposite of parse_is_page, and is only called   *
 * from same. It takes the linked list of "NotPageType" and applies     *
 * and RE against a log line built from that list.                      *
 *                                                                      *
 * A successfull match means that this line is NOT a page               *
 *                                                                      *
 * Arguments:                                                           *
 * -----------                                                          *
 * char *url      -  A URL field, typically: log_rec.url                *
 *                                                                      *
 * Returns:                                                             *
 * -----------                                                          *
 * Boolean.  True if, yes this is a page                                *
 *           False is not a page. ie Successfull match.                 *
 *                                                                      *
 ************************************************************************/
bool
parse_check_not_page(char *url)
{
    char regex_page[MAX_RE_LENGTH] = "";        /* Hold the PAGE RE */
    static pcre *cmp_regex_page = NULL;         /* NotPage compiled RE */

    const char *error;                          /* RE error pointer, offset */
    int erroffset;                              /* RE error value */
    int str_length, tmp_length;
    int rc;                                     /* RE Check return value */
    static int max_type_length = 0;
    char *str_start = url;

    LISTPTR lptr = not_page_type;

    /* Compile both RegEx's */
    if (cmp_regex_page == NULL) {
        /* Build the RegEx first, loop thru the PageType list & reverse */
        strcat(regex_page, "\\.(");
        while (lptr != NULL) {
            strcat(regex_page, lptr->string);
            str_length = strlen(lptr->string);
            if (str_length > max_type_length) {
                max_type_length = str_length;
            }
            lptr = lptr->next;
            if (lptr != NULL) {
                strcat(regex_page, "|");
            }
        }
        strcat(regex_page, ")$");
        VPRINT(VERBOSE2, "PCRE: New NotPAGE RegEx: '%s',  Max: %d\n", regex_page, max_type_length);

        /* Compile the RegEx */
        cmp_regex_page = pcre_compile(regex_page, 0, &error, &erroffset, NULL);
        VPRINT(VERBOSE2, "PCRE: Compile PAGE%s", "\n");
        if (cmp_regex_page == NULL) {
            re_compile_failed(erroffset, error, regex_page);
        }
        max_type_length++;                      /* Increase by 1 for starting '.' */
    }

    str_length = strlen(url);
    if (str_length < max_type_length) {
        tmp_length = str_length;
    } else {
        tmp_length = max_type_length;
        str_start = url + str_length - max_type_length;
    }
    VPRINT(VERBOSE4, "  Was: '%s', Is: %s\n", url, str_start);

    rc = pcre_exec(cmp_regex_page, NULL, str_start, tmp_length, 0, 0, NULL, 0);
    /* check for RE matching */
    if (rc >= 0) {
        /* Have matched! */
        return (false);
    }
    return (true);
}

/************************************************************************
 * parse_is_page                                                        *
 *                                                                      *
 * Given a URL (field from a log line)                                  *
 *   determine if this URL is a page or not                             *
 *                                                                      *
 * Logic:                                                               *
 * -----------                                                          *
 * The RegEx and URL provided are reversed for efficiency - we only     *
 *  want to match the end of a URL, not the entire thing.               *
 * Firstly builds the RegEx. Does this by reversing the provided        *
 *  PageType config options, and wrappering appropriate RE around.
 *                                                                      *
 * Arguments:                                                           *
 * -----------                                                          *
 * char *url      -  A URL field, typically: log_rec.url                *
 *                                                                      *
 * Returns:                                                             *
 * -----------                                                          *
 * Boolean.  True if, yes this is a page                                *
 *           False in all other cases. Including invalid args.          *
 *                                                                      *
 ************************************************************************/

bool
parse_is_page(char *url)
{
    char regex_page[MAX_RE_LENGTH] = "";        /* Hold the PAGE RE */
    static pcre *cmp_regex_page = NULL;         /* Page compiled RE */

    const char *error;                          /* RE error pointer, offset */
    int erroffset;                              /* RE error value */
    int str_length;
    int rc;                                     /* RE Check return value */
    char reverse[MAXURL] = "";
    int i, j = 0;
    LISTPTR lptr;

    if (not_page_type != NULL) {
        return (parse_check_not_page(url));
    }

    lptr = page_type;

    /* Compile both RegEx's */
    if (cmp_regex_page == NULL) {
        /* Build the RegEx first, loop thru the PageType list & reverse */
        strcat(regex_page, "^(\\/|(");
        while (lptr != NULL) {
            str_length = (int) strlen(lptr->string);
            j = 0;
            for (i = str_length - 1; i >= 0; i--) {
                if (lptr->string[i] == '*') {
                    reverse[j] = '.';
                    j++;
                }
                reverse[j] = lptr->string[i];
                j++;
            }
            reverse[j] = '\0';
            strcat(regex_page, reverse);
            lptr = lptr->next;
            if (lptr != NULL) {
                strcat(regex_page, "|");
            }
        }
        strcat(regex_page, ")\\.|[^./]+\\/)");
        VPRINT(VERBOSE2, "PCRE: New PAGE RegEx: '%s'\n", regex_page);

        /* Compile the RegEx */
        cmp_regex_page = pcre_compile(regex_page, 0, &error, &erroffset, NULL);
        VPRINT(VERBOSE2, "PCRE: Compile PAGE%s", "\n");
        if (cmp_regex_page == NULL) {
            re_compile_failed(erroffset, error, regex_page);
        }
    }

    str_length = (int) strlen(url);
    j = 0;
    for (i = str_length - 1; i >= 0; i--) {
        /* Use pointer math - faster at loss of clarity */
        *(reverse + j) = *(url + i);
        j++;
    }
    *(reverse + j) = '\0';                      /* Probably not needed as we provide the length... */

    rc = pcre_exec(cmp_regex_page, NULL, reverse, str_length, 0, 0, NULL, 0);
    /* check for RE matching */
    if (rc >= 0) {
        /* Have matched! */
        return (true);
    }
    return (false);
}                                               /* parse_is_page */


/************************************************************************
 * parse_is_bookmark                                                    *
 *                                                                      *
 * Given a URL and a BrowserID (respective fields from a log line)      *
 *   determine if these entries indicate a Bookmark being made          *
 *                                                                      *
 * Relies on the action taken by MSIE to grab a favicon.ico on          *
 *   bookmarking                                                        *
 *                                                                      *
 * Arguments:                                                           *
 * -----------                                                          *
 * char *url      -  A URL field, typically: log_rec.url                *
 * char *browser  -  A Browser ID field, typically: log_rec.agent       *
 *                                                                      *
 * Returns:                                                             *
 * -----------                                                          *
 * Boolean.  True if, yes this is a bookmark                            *
 *           False in all other cases. Including invalid args.          *
 *                                                                      *
 ************************************************************************/

bool
parse_is_bookmark(char *url, char *browser)
{
    char regex_favicon[MAX_RE_LENGTH] = "^\\/favicon\\.ico$";   /* Hold the FAVICON RE */
    char regex_msie[MAX_RE_LENGTH] = "MSIE";    /* Hold the MS IE RE */

    static pcre *cmp_regex_favicon = NULL;      /* FAVICON compiled RE */
    static pcre *cmp_regex_msie = NULL;         /* MSIE compiled RE */

    int ovector[OVECCOUNT];                     /* RE substring offsets array */
    const char *error;                          /* RE error pointer, offset */
    int erroffset;                              /* RE error value */
    int str_length;
    int rc;                                     /* RE Check return value */

    /* Compile both RegEx's */
    if (cmp_regex_favicon == NULL) {
        cmp_regex_favicon = pcre_compile(regex_favicon, 0, &error, &erroffset, NULL);
        VPRINT(VERBOSE2, "PCRE: Compile FAVICON%s", "\n")
            if (cmp_regex_favicon == NULL) {
            ERRVPRINT(VERBOSE0, "%s\n", "Failed to compile FAVICON RegEx!");
            exit(1);
        }

        cmp_regex_msie = pcre_compile(regex_msie, 0, &error, &erroffset, NULL);
        VPRINT(VERBOSE2, "PCRE: Compile MSIE%s", "\n")
            if (cmp_regex_favicon == NULL) {
            ERRVPRINT(VERBOSE0, "%s\n", "Failed to compile MSIE RegEx!");
            exit(1);
        }
    }

    str_length = (int) strlen(url);
    rc = pcre_exec(cmp_regex_favicon, NULL, url, str_length, 0, 0, ovector, OVECCOUNT);
    /* check for RE matching */
    if (rc >= 0) {
        /* Have matched! */
        str_length = (int) strlen(browser);
        rc = pcre_exec(cmp_regex_msie, NULL, browser, str_length, 0, 0, ovector, OVECCOUNT);
        if (rc >= 0) {
            return (true);
        }                                       /* Don't bother with normal RE failure checks - too many false alarms */
    }

    return (false);
}                                               /* parse_is_bookmark */


/************************************************************************
 * re_check_errors                                                      *
 *                                                                      *
 * After an RE check, deal with any errors                              *
 * err: value returned from pcre_exec                                   *
 * str_ptr: String that failed to match                                 *
 ************************************************************************/
void
re_check_errors(int err)
{

    /* Matching failed: handle error cases */
    switch (err) {
    case PCRE_ERROR_NOMATCH:
        ERRVPRINT(VERBOSE1, "WARN: No Regular Expression Match. %s", "");
        break;
        /*  Leave out the more explicit failure messages - we show the number, so can be found.
           case PCRE_ERROR_NULL:
           case PCRE_ERROR_BADOPTION:
           case PCRE_ERROR_BADMAGIC:
           case PCRE_ERROR_UNKNOWN_NODE:
           case PCRE_ERROR_NOMEMORY:
           case PCRE_ERROR_NOSUBSTRING:
           case PCRE_ERROR_MATCHLIMIT:
           case PCRE_ERROR_CALLOUT:
           case PCRE_ERROR_BADUTF8:
           case PCRE_ERROR_BADUTF8_OFFSET:
           case PCRE_ERROR_PARTIAL:
           case PCRE_ERROR_BAD_PARTIAL:
           case PCRE_ERROR_INTERNAL:
           case PCRE_ERROR_BADCOUNT:
         */
    default:
        ERRVPRINT(VERBOSE1, "WARN: Regular Expression Error: %d : %s", err, "");
        break;
    }
}


/************************************************************************
 * re_compile_failed                                                    *
 *                                                                      *
 * Display a failed RE Compile & where                                  *
 * FATAL failure. Will exit the run.                                    *
 ************************************************************************/
void
re_compile_failed(int err, const char *err_offset, char *re_str)
{
    ERRVPRINT(VERBOSE0, "FATAL ERROR! PCRE compilation failed at offset %d: %s\nUsing RegExp: %s\n", err, err_offset, re_str);
    exit(1);                                    /* FIXME - table of exit codes! */
}


/************************************************************************
 * error_substring_extract                                              *
 *                                                                      *
 * Display a failed substring extraction                                *
 * Error Only, as this should have failed the RE                        *
 ************************************************************************/
void
error_substring_extract(int substr_idx, int err)
{
    ERRVPRINT(VERBOSE1, "ERROR! Failed to extract substring: %d", substr_idx);
    switch (err) {
    case PCRE_ERROR_NOMEMORY:
        ERRVPRINT(VERBOSE1, "  PCRE: Insufficient Memory.%s  ", "");
        break;
    case PCRE_ERROR_NOSUBSTRING:
        ERRVPRINT(VERBOSE1, "  PCRE: Substring doesn't exist.%s  ", "");
        break;
    default:
        ERRVPRINT(VERBOSE1, "  Unknown PCRE Error: %d  ", err);
        break;
    }
}


/************************************************************************
 ************************************************************************
 *                      END OF FILE                                     *
 ************************************************************************/
